﻿/*
 *  TOPPERS/ECNL Prototyping tool
 *
 *  Copyright (C) 2017 Cores Co., Ltd. Japan
 *
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  @(#) $Id$
 */
using System;
using System.Collections.Generic;
using Bridge;
using Bridge.Html5;
using Bridge.jQuery2;

namespace WebMrbc
{
	public class ClassSelectorView
	{
		Collection<IClassWorkspace> m_Collection;
		jQuery el;
		string templateText;
		IClassWorkspace m_ClassWorkspace;

		internal IClassWorkspace Current { get { return m_ClassWorkspace; } }

		internal event EventHandler<EventArgs> Selected;
		internal event EventHandler<ItemRemovedEventArgs> Removed;
		internal event EventHandler<EventArgs> MarkClicked;

		internal void SetCollection(Collection<IClassWorkspace> collection)
		{
			el = jQuery.Select("#class-selector-tab");

			m_Collection = collection;
			collection.OnAdd += OnChange;
			collection.OnRemove += OnChange;
			collection.OnReset += OnChange;
			collection.OnChange += OnChange;

			if (collection.Length > 0)
				m_ClassWorkspace = collection.At(0);

			templateText = jQuery.Select("#class-selector-template").Text();
			jQuery.Select("#add-celltype-button").Click(null, new Action<jQueryMouseEvent>(OnAddBtnClick));
			Render();
		}

		void Render()
		{
			var charsEl = jQuery.Select("#class-selector-celltype-set");
			charsEl.Children().Remove();
			foreach (var item in m_Collection) {
				var html = new jQuery(item.Template(templateText));
				charsEl.Append(html);
				var selectedMark = html.Find("a.selected-mark");
				if (m_ClassWorkspace == item)
					selectedMark.Show();
				else
					selectedMark.Hide();
				selectedMark.Click(item, new Action<jQueryMouseEvent>(OnSelectedMarkClick));
				html.Find("a.celltype").Click(item, new Action<jQueryMouseEvent>(OnSelectBtnClick));
				html.Find("a.modify-button").Click(item, new Action<jQueryMouseEvent>(OnModifyBtnClick));
				var removeButton = html.Find("a.remove-button");
				removeButton.Click(item, new Action<jQueryMouseEvent>(OnRemoveBtnClick));
				if (item.IsPreset())
					removeButton.Hide();
				var img = html.Find("img");
				img.On("dragstart", new Action<jQueryMouseEvent>((e) => {
					e.PreventDefault();
				}));
			}
		}

		private void OnSelectedMarkClick(jQueryMouseEvent obj)
		{
			MarkClicked?.Invoke(this, EventArgs.Empty);
		}

		internal void SelectClassWorkspace(IClassWorkspace model)
		{
			jQuery html, selectedMark;
			var charsEl = jQuery.Select("#class-selector-celltype-set");

			if (m_ClassWorkspace != null) {
				html = charsEl.Find("#" + m_ClassWorkspace.Identifier);
				selectedMark = html.Find("a.selected-mark");
				selectedMark.Hide();
				m_ClassWorkspace.Inactivate();
			}

			m_ClassWorkspace = model;
			if ((m_ClassWorkspace == null) && (m_Collection.Length > 0)) {
				m_ClassWorkspace = m_Collection.At(0);
			}

			if (m_ClassWorkspace != null) {
				m_ClassWorkspace.Activate();

				html = charsEl.Find("#" + m_ClassWorkspace.Identifier);
				selectedMark = html.Find("a.selected-mark");
				selectedMark.Show();
			}

			Selected?.Invoke(this, EventArgs.Empty);
		}

		void RemoveClassWorkspace(IClassWorkspace item)
		{
			if (item.IsPreset())
				return;

			if (m_ClassWorkspace == item)
				m_ClassWorkspace = null;

			m_Collection.Remove(item);

			Removed?.Invoke(this, new ItemRemovedEventArgs(item));
		}

		private void OnSelectBtnClick(jQueryMouseEvent e)
		{
			var item = (IClassWorkspace)e.Data;
			e.PreventDefault();
			if (m_ClassWorkspace != item) {
				this.SelectClassWorkspace(item);
			}
		}

		private void OnModifyBtnClick(jQueryMouseEvent e)
		{
			var item = (IClassWorkspace)e.Data;
			e.PreventDefault();
			m_ClassWorkspace = item;
			m_ClassWorkspace.OpenModifyView((ok) => {
				Render();
			});
		}

		private void OnAddBtnClick(jQueryMouseEvent e)
		{
			e.PreventDefault();

			App.NewItem((item) => {
				item.OpenModifyView((ok) => {
					if (ok) {
						m_Collection.Add(item);
						SelectClassWorkspace(item);
					}
					else {
						App.RemoveItem(item);
					}
				});
			});
		}

		private void OnRemoveBtnClick(jQueryMouseEvent e)
		{
			var item = (IClassWorkspace)e.Data;
			e.PreventDefault();
			m_ClassWorkspace = null;
			RemoveClassWorkspace(item);
		}

		void OnChange(object sender, EventArgs e)
		{
			Render();
		}
	}

	class ItemRemovedEventArgs : EventArgs
	{
		public IClassWorkspace Item { get; private set; }

		public ItemRemovedEventArgs(IClassWorkspace item)
		{
			Item = item;
		}
	}
}
